/**
 * apeman bud for ci scripts.
 * @memberof module:apeman-bud-ci/lib
 * @function setup
 * @param {object} params - Template params.
 * @returns {object} - Coz bud configuration.
 */

"use strict";

const argx = require('argx'),
    async = require('async'),
    arrayreduce = require('arrayreduce'),
    apemanfile = require('apemanfile'),
    _bud = require('./_bud');

/** @lends setup */
function setup(params) {
    let args = argx(arguments);
    params = args.shift('object');
    return function bud(callback) {
        async.waterfall([
            (callback) => {
                apemanfile.load(params.configuration, callback);
            },
            (apemanfile, callback) => {
                let $children = apemanfile.$children || {};
                let $tasks = apemanfile.$tasks || {};
                let bud = Object.keys($tasks).map(function (name) {
                    return _bud(name, params);
                });
                bud = bud.concat(
                    Object.keys($children).map(function (key) {
                        let $child = $children[key];
                        let $tasks = $child.$tasks || {};
                        let cwd = params.cwd;
                        return Object.keys($tasks).map((name) => {
                            return _bud([key, name].join('#'), {
                                cwd: cwd ? cwd + '/..' : null
                            });
                        });
                    })
                ).reduce(arrayreduce.arrayConcat(), []);
                callback(null, bud);
            }
        ], callback);
    };
}

setup.byNames = function (names, params) {
    return names.map((name) => {
        return _bud(name, params || {});
    });
};

module.exports = setup;
